use based::request::api::{FallibleApiResponse, api_error};
use rocket::tokio::io::AsyncReadExt;
use rocket::{FromForm, State, post};
use serde_json::json;

use pacco::pkg::Package;
use pacco::pkg::arch::Architecture;

// /pkg/<repo>/<arch>/<pkg_name>
// /pkg/<repo>/<arch>/

use rocket::form::Form;
use rocket::fs::TempFile;

use crate::config::Config;

#[derive(FromForm)]
pub struct PkgUpload<'r> {
    name: String,
    arch: String,
    version: String,
    pkg: TempFile<'r>,
    sig: Option<TempFile<'r>>,
}

pub async fn tmp_file_to_vec<'r>(tmp: &TempFile<'r>) -> Vec<u8> {
    let mut buf = Vec::with_capacity(tmp.len() as usize);
    tmp.open()
        .await
        .unwrap()
        .read_to_end(&mut buf)
        .await
        .unwrap();
    buf
}

#[post("/pkg/<repo>/upload", data = "<upload>")]
pub async fn upload_pkg(
    repo: &str,
    upload: Form<PkgUpload<'_>>,
    user: based::auth::APIUser,
    config: &State<Config>,
) -> FallibleApiResponse {
    // TODO : Permission System
    if !user.0.is_admin() {
        return Err(api_error("Forbidden"));
    }

    if config.is_mirrored_repo(repo) {
        return Err(api_error("This repository is a mirror."));
    }

    let pkg = Package::new(
        repo,
        Architecture::parse(&upload.arch).ok_or_else(|| api_error("Invalid architecture"))?,
        &upload.name,
        &upload.version,
    );

    pkg.save(
        tmp_file_to_vec(&upload.pkg).await,
        if let Some(sig) = &upload.sig {
            Some(tmp_file_to_vec(sig).await)
        } else {
            None
        },
    );

    Ok(json!({"ok": format!("Added '{}' to '{}'", pkg.file_name(), repo)}))
}
